#!/bin/bash

#generate the random operator for orangutan
#

#options
MAXW=320
MAXH=480
STEPS=100000
tap_dur=200
drag_dur=50
cmd_delay=500

usage()
{
    echo "Usage: $(basename $0) [fwvga|wvga|hvga(default)|qvga] [--steps num] [--help]"
    exit $1
}

while [ $# -gt 0 ]
do
    case $1 in
    hvga|320x480)
        #default, do nothing
        ;;
    fwvga|480x854)
        MAXW=480
        MAXH=854
        ;;
    wvga|480x800)
        MAXW=480
        MAXH=800
        ;;
    qvga|240x320)
        MAXW=240
        MAXH=320
        ;;
    540x960)
        MAXW=540
        MAXH=960
        ;;
    720x1280)
        MAXW=720
        MAXH=1280
        ;;
    --steps)
        shift
        if [[ "$1" == *[!0-9]* ]]
        then
            echo "Steps must be a numeric."
            usage 1
        else
            [ $1 -lt $STEPS ] && STEPS=$1
        fi
        ;;
    --help | -h)
        usage 0
        ;;
    -*)
        echo "Unrecognized option $1"
        usage 1
        ;;
    *)
        break
        ;;
    esac

    shift
done

#if you want to change the rate if each command
#you can simply set more the command in the array
#such as more tap and no sleep
#CMDS=(tap tap tap tap tap tap drag drag pinch)
CMDS=(tap drag pinch)

for((i=0;i<$STEPS;i++))
do
    cmd=${CMDS[$(( $RANDOM % ${#CMDS[@]} ))]}
    case $cmd in
    tap)
        #tap x y step duration
        x=$(( $RANDOM % $MAXW ))
        y=$(( $RANDOM % $MAXH ))
        tap=$(( $RANDOM % 3 + 1 ))
        echo $cmd $x $y $tap $tap_dur 
        ;;
    drag)
        #drag x1 y1 x2 y2 step duration
        x1=$(( $RANDOM % $MAXW ))
        y1=$(( $RANDOM % $MAXH ))
        x2=$(( $RANDOM % $MAXW ))
        y2=$(( $RANDOM % $MAXH ))
        step=$(( $RANDOM % 10 + 10 ))
        echo $cmd $x1 $y1 $x2 $y2 $step $drag_dur 
        ;;
    pinch)
        #drag x1 y1 x2 y2 a1 b1 a2 b2 step duration
        x1=$(( $RANDOM % $MAXW ))
        y1=$(( $RANDOM % $MAXH ))
        x2=$(( $RANDOM % $MAXW ))
        y2=$(( $RANDOM % $MAXH ))
        a1=$(( $RANDOM % $MAXW ))
        b1=$(( $RANDOM % $MAXH ))
        a2=$(( $RANDOM % $MAXW ))
        b2=$(( $RANDOM % $MAXH ))
        step=$(( $RANDOM % 10 + 10 ))
        echo $cmd $x1 $y1 $x2 $y2 $a1 $b1 $a2 $b2 $step $drag_dur 
        ;;
    *)
        break
        ;;
    esac

    echo sleep $cmd_delay
done
